@@ -2110,10 +2110,16 @@ E  unsigned char *FDECL(e2sj, (unsigned char *));
 E  unsigned char *FDECL(sj2e, (unsigned char *));
 
 E  char *NDECL(remote_err);
-E  int NDECL(connect_scoreserver);
+E  int FDECL(connect_scoreserver, (int));
 E  int FDECL(soc_read, (int, char *, int));
 E  int FDECL(soc_wait, (int));
 
+E  void FDECL(set_email, (char *));
+E  void FDECL(set_homeurl, (char *));
+
+E  char *NDECL(get_email);
+E  char *NDECL(get_homeurl);
+
 #undef E
 
 #endif /* EXTERN_H */
