@@ -128,7 +128,7 @@ static struct Bool_Opt
 #else
 	{"page_wait", (boolean *)0, FALSE},
 #endif
-	{"perm_invent", &flags.perm_invent, TRUE /*JP FALSE */},
+	{"perm_invent", &flags.perm_invent, FALSE},
 #ifdef MAC
 	{"popup_dialog", &flags.popup_dialog, FALSE},
 #else
@@ -139,6 +139,9 @@ static struct Bool_Opt
 #else
 	{"preload_tiles", (boolean *)0, FALSE},
 #endif
+#ifdef RADAR
+	{ "radar", &flags.radar, TRUE},
+#endif
 #if defined(MICRO) && !defined(AMIGA)
 	{"rawio", &iflags.rawio, FALSE},
 #else
@@ -292,7 +295,9 @@ static struct Comp_Opt
 #endif
 	{ "windowtype", "Ѥ륦ɥƥ,", WINTYPELEN },
 /*JP*/
-	{ "kcode","üδ,", 4 },
+	{ "kcode", "üδ,", 4 },
+/*	{ "email", "ʤE-Mailɥ쥹,", 50 },*/
+	{ "home", "ʤΥۡڡURL,", 128 },
 	{ NULL, NULL }
 };
 
@@ -1201,6 +1206,24 @@ goodfruit:
 	  }
 	  return;
 	}
+/*
+	fullname = "email";
+	if (match_optname(opts, fullname, 3, TRUE)) {
+	    if (negated) 
+		bad_negation(fullname, FALSE);
+	    else if ((op = string_for_env_opt(fullname, opts, FALSE)) != 0)
+		set_email(op);
+	    return;
+	}
+*/
+	fullname = "homeurl";
+	if (match_optname(opts, fullname, 5, TRUE)) {
+	    if (negated) 
+		bad_negation(fullname, FALSE);
+	    else if ((op = string_for_env_opt(fullname, opts, FALSE)) != 0)
+		set_homeurl(op);
+	    return;
+	}
 
 	/* scores:5t[op] 5a[round] o[wn] */
 	if (match_optname(opts, "scores", 4, TRUE)) {
@@ -1662,6 +1685,9 @@ doset()
 	doset_add_menu(tmpwin, "videocolors", buf, 0);
 #endif /* VIDEOSHADES */
 	doset_add_menu(tmpwin, "windowtype", windowprocs.name, 0);
+/*JP*/
+/*	doset_add_menu(tmpwin, "email", get_email(), 0);*/
+	doset_add_menu(tmpwin, "homeurl", get_homeurl(), 0);
 	/* modifiable compounds */
 	doset_add_menu(tmpwin, "disclose",
 		       flags.end_disclose[0] ? flags.end_disclose : "all",
