@@ -1702,7 +1702,7 @@ dodip()
 	    (mixture = mixtype(obj, potion)) != 0) {
 		/* with multiple merged potions, we should split off one and
 		   just clear it, but clearing them all together is easier */
-		boolean more_than_one = potion->quan > 1L;
+/*JP		boolean more_than_one = potion->quan > 1L;*/
 		potion->otyp = mixture;
 		potion->blessed = 0;
 		if (mixture == POT_WATER)
