@@ -527,8 +527,8 @@ int how;
 	(void) fclose(rfile);
 	unlock_file(RECORD);
 	free_ttlist(tt_head);
-
   showwin:
+
 	if (flags.toptenwin && !done_stopprint) display_nhwindow(toptenwin, 1);
   destroywin:
 	if (!t0_used) dealloc_ttentry(t0);
@@ -562,7 +562,7 @@ boolean re;
 {
 	boolean second_line = TRUE;
 	char linebuf[BUFSZ];
-	char *bp, hpbuf[24], linebuf3[BUFSZ];
+	char *bp, hpbuf[24]/*JP, linebuf3[BUFSZ]*/;
 	int hppos, lngr;
 /*JP*/
 	char who[BUFSZ];
@@ -582,10 +582,13 @@ boolean re;
 
 /*JP	Sprintf(eos(linebuf), " %10ld  %.10s", t1->points, t1->name);*/
 /*JP	Sprintf(eos(linebuf), "-%c ", t1->plchar);*/
+	bp = index(pl_classes, t1->plchar);
 	Sprintf(who, " %10ld  %s", t1->points, 
-		jtrns_mon(pl_character, flags.female));
+		jtrns_mon(roles[bp - pl_classes], t1->sex == 'F'));
+/*JP
+		jtrns_mon(pl_character, flags.female));*/
 	Sprintf(eos(who), "%s(%s)", t1->name,
-		flags.female ? "" : "");
+		t1->sex == 'F' ? "" : "");
 /*JP*/
 	jdeath = t1->death;
 	if (!strncmp(jdeath, "", 12))
@@ -649,7 +652,7 @@ boolean re;
 #endif /*JP*/
 
 	    if (t1->deathdnum == astral_level.dnum) {
-		const char *arg, *fmt = " on the Plane of %s";
+	        const char *arg/*JP, *fmt = " on the Plane of %s"*/;
 
 		switch (t1->deathlev) {
 		case -5:
@@ -712,9 +715,9 @@ boolean re;
 	  FILE *report;
 	  int sd;
 
-	  sd = connect_scoreserver();
+	  sd = connect_scoreserver(SCORE_TIMEOUT);
 	  if(sd < 0){
-	    /*	    putstr(endwin, 0, "ERROR\n");*/;
+	      raw_printf("Report: %s", remote_err());
 	  }
 	  else{
 	    report = fdopen(sd, "w+");
@@ -750,6 +753,8 @@ boolean re;
 	    fprintf(report, "gold: %d\r\n", u.ugold);
 	    fprintf(report, "moves: %d\r\n", moves);
 	    fprintf(report, "dying message: %s\r\n", linebuf);
+/*	    fprintf(report, "email: %s\r\n", get_email());*/
+	    fprintf(report, "homeurl: %s\r\n", get_homeurl());
 	    fprintf(report, ".\r\n");
 	    fflush(report);
 
