@@ -548,7 +548,7 @@ register int thrown;
 			  if (thrown) obfree(o,(struct obj *)0); \
 			  else useup(o); \
 			  o = (struct obj *)0; }	/* now gone */
-			long cnt = obj->quan;
+			/*JP long cnt = obj->quan;*/
 
 			tmp = 1;		/* nominal physical damage */
 			get_dmg_bonus = FALSE;
