@@ -1,21 +1,26 @@
 #!/bin/sh
 #	SCCS Id: @(#)nethack.sh	3.2	90/02/26
 
-# JP
-# set LANG, LC_CTYPE, NETHACKOPTIONS here
-#
-LANG=ja_JP.EUC
-LC_CTYPE=ja_JP.EUC
-NETHACKOPTIONS=
-
-export LANG
-export LC_CTYPE
-export NETHACKOPTIONS
-
 HACKDIR=/usr/games/lib/nethackdir
 HACK=$HACKDIR/nethack
 MAXNROFPLAYERS=20
 
+# JP
+# set LC_CTYPE, NETHACKOPTIONS etc..
+#
+if [ "X$LC_CTYPE" = "X" -o "X$LC_CTYPE" = "XC" ] ; then
+	LC_CTYPE=ja_JP.EUC
+	export LC_CTYPE
+fi
+if [ "X$NETHACKOPTIONS" = "X" ] ; then
+	NEXTHACKOPTIONS=
+	export NETHACKOPTIONS
+fi
+if [ "X$APPRESDIR" = "X" ] ; then
+	XAPPLRESDIR=$HACKDIR
+	export XAPPLRESDIR
+fi
+
 # see if we can find the full path name of PAGER, so help files work properly
 # assume that if someone sets up a special variable (HACKPAGER) for NetHack,
 # it will already be in a form acceptable to NetHack
