@@ -162,7 +162,6 @@ static int FDECL(input_event, (int));
 static void FDECL(win_visible, (Widget,XtPointer,XEvent *,Boolean *));
 static void NDECL(init_standard_windows);
 
-
 /*
  * Local variables.
  */
@@ -909,6 +908,9 @@ static XtActionsRec actions[] = {
     {"key_dismiss_text",key_dismiss_text},/* key action for text widget */
 #ifdef GRAPHIC_TOMBSTONE
     {"rip_dismiss_text",rip_dismiss_text},/* action for rip in text widget */
+#endif
+#ifdef RADAR
+    {"dismiss_radar",	dismiss_radar},
 #endif
     {"menu_key",	menu_key},	/* action for menu accelerators */
     {"yn_key",		yn_key},	/* action for yn accelerators */
@@ -935,6 +937,12 @@ static XtResource resources[] = {
       XtOffset(AppResources *,double_tile_size), XtRString, "False" },
     { "tile_file", "Tile_file", XtRString, sizeof(String),
       XtOffset(AppResources *,tile_file), XtRString, "" },
+/*JP*/
+    { "tile_width", "Tile_width", XtRInt, sizeof(int),
+      XtOffset(AppResources *,tile_width), XtRString, "32" },
+    { "tile_height", "Tile_height", XtRInt, sizeof(int),
+      XtOffset(AppResources *,tile_height), XtRString, "32" },
+/*JP*/
     { "icon", "Icon", XtRString, sizeof(String),
       XtOffset(AppResources *,icon), XtRString, "nh72" },
     { "message_lines", "Message_lines", XtRInt, sizeof(int),
@@ -1360,7 +1368,7 @@ X11_display_file(str, complain)
     Cardinal num_args;
     Widget popup, dispfile;
     Position top_margin, bottom_margin, left_margin, right_margin;
-    XFontStruct *fs;
+/*JP    XFontStruct *fs;*/
     int new_width, new_height;
 #define LLEN 128
     char line[LLEN];
