@@ -76,13 +76,14 @@ static void FDECL(display_cursor, (struct xwindow *));
 #define RADAR_WIDTH	(80 * 4)
 #define RADAR_HEIGHT	(24 * 4)
 
-static int	use_radar = 1;
+#define USE_RADAR	(map_info->is_tile)
+static int	radar_is_popuped;
+static int	radar_is_initialized;
 
 static Widget	radar;
 static int	radar_x, radar_y;
 static int	radar_w, radar_h;
 
-static int	radar_flg;
 static Widget	radar_popup;
 static Window	radar_window;
 static Pixmap	radar_pixmap;
@@ -90,6 +91,7 @@ static Pixmap	radar_pixmap2;
 static GC	radar_gc_black;
 static GC	radar_gc_white;
 static GC	radar_gc[16];
+
 static XColor	radar_color[16] = {
   {0, 0x0000, 0x0000, 0x0000,},	/* black */
   {0, 0xffff, 0xffff, 0xffff,},	/* white */
@@ -100,7 +102,7 @@ static XColor	radar_color[16] = {
   {0, 0xffff, 0x0000, 0xffff,},	/* magenta */
   {0, 0x8000, 0x8000, 0x8000,},	/* gray */
   {0, 0xffff, 0x8000, 0x0000,},	/* orange */
-  {0, 0x2000, 0x8000, 0x8000,},	/* dark green */
+  {0, 0x0000, 0x8000, 0x0000,},	/* dark green */
   {0, 0x0000, 0xffff, 0xffff,},	/* cyan */
 };
 enum {
@@ -119,9 +121,24 @@ enum {
 
 #endif
 
+/*ITA*/
+struct pxm_slot_t {
+    int fg;
+    int bg;
+    int age;
+    Pixmap pixmap;
+};
+#define MAX_PXM_SLOTS 100
+    struct pxm_slot_t pxm_slot[MAX_PXM_SLOTS]; 
+/*ITA*/
+
+
 /* Global functions ======================================================== */
 
 void
+/*JP
+X11_print_glyph(window, x, y, glyph)
+*/
 X11_print_glyph(window, x, y, glyph)
     winid window;
     xchar x, y;
@@ -139,15 +156,14 @@ X11_print_glyph(window, x, y, glyph)
 
     if (map_info->is_tile) {
 	unsigned short *t_ptr;
-
 	t_ptr = &map_info->mtype.tile_map->glyphs[y][x];
 
 	if (*t_ptr != glyph) {
 	    *t_ptr = glyph;
 	    update_bbox = TRUE;
-	} else
-	    update_bbox = FALSE;
-
+	}
+	else
+	  update_bbox = FALSE;
     } else {
 	uchar			ch;
 	register int		offset;
@@ -252,10 +268,19 @@ void X11_cliparound(x, y) int x, y; { }
  * or just keep it on a per-window basis.
  */
 Pixmap tile_pixmap = None;
+Pixmap tile_clipmask = None;
+GC     tile_gc;
+XpmImage tile_image;
+
+#define	TILE_WIDTH	appResources.tile_width
+#define	TILE_HEIGHT	appResources.tile_height
+int	TILE_PER_COL;
+/*
+#define TILE_PER_COL	32
 static int tile_width;
 static int tile_height;
 static int tile_count;
-static XImage *tile_image = 0;
+*/
 
 /*
  * This structure is used for small bitmaps that are used for annotating
@@ -302,28 +327,44 @@ void
 post_process_tiles()
 {
     Display *dpy = XtDisplay(toplevel);
-    unsigned int width, height;
+/*    unsigned int width, height;*/
+    Colormap cmap;
+    XpmAttributes attributes;
+/*    int errorcode;*/
+    Arg args[16];
+    XGCValues val;
 
-    height = tile_height * tile_count;
-    width  = tile_width;
+#ifdef USE_XPM
+    if(tile_image.data){
+      XtSetArg(args[0], XtNcolormap, &cmap);
+      XtGetValues(toplevel, args, ONE);
+      
+      attributes.valuemask = XpmCloseness | XpmColormap;
+      attributes.colormap = cmap;
+      attributes.closeness = 25000;
 
-    if (tile_image == 0) return;	/* no tiles */
+      XpmCreatePixmapFromXpmImage(
+		dpy,
+		XtWindow(toplevel),
+		&tile_image,
+		&tile_pixmap,
+		&tile_clipmask,
+		&attributes
+		);
 
-    tile_pixmap = XCreatePixmap(dpy, XtWindow(toplevel),
-			width,
-    			height,
-			DefaultDepth(dpy, DefaultScreen(dpy)));
-
-    XPutImage(dpy, tile_pixmap,
-	DefaultGC(dpy, DefaultScreen(dpy)),
-	tile_image,
-	0,0, 0,0,		/* src, dest top left */
-	width,
-	height);
-
-    XDestroyImage(tile_image);	/* data bytes free'd also */
-    tile_image = 0;
+      val.function = GXcopy;
+      val.clip_mask = tile_clipmask;
 
+      tile_gc = XCreateGC(
+		dpy,
+		XtWindow(toplevel),
+		GCFunction | GCClipMask,
+		&val
+		);
+		
+      XpmFreeXpmImage(&tile_image);
+    }
+#endif
     init_annotation(&pet_annotation,
 	appResources.pet_mark_bitmap, appResources.pet_mark_color);
 }
@@ -368,31 +409,17 @@ init_tiles(wp)
 
 #ifdef USE_XPM
     {
-        Colormap cmap;
-	char buf[BUFSIZ];
-	XpmAttributes attributes;
 	int errorcode;
-	Arg args[16];
+	char buf[BUFSIZ];
 
-	XtSetArg(args[0], XtNcolormap, &cmap);
-	XtGetValues(toplevel, args, ONE);
+        errorcode = XpmReadFileToXpmImage(
+		    appResources.tile_file,
+		    &tile_image,
+		    NULL
+		    );
 
-	attributes.valuemask = XpmCloseness | XpmColormap;
-	attributes.colormap = cmap;
-	attributes.closeness = 25000;
-
-	errorcode=XpmReadFileToImage(dpy,appResources.tile_file,&tile_image,0,&attributes);
-
-	if (errorcode == XpmColorFailed) {
-	    Sprintf(buf, "Insufficient colors available to load %s.",appResources.tile_file);
-	    X11_raw_print(buf);
-	    X11_raw_print("Try closing other colorful applications and restart.");
-	    X11_raw_print("Attempting to load with inferior colors.");
-	    attributes.closeness = 50000;
-	    errorcode=XpmReadFileToImage(dpy,appResources.tile_file,&tile_image,0,&attributes);
-	}
 	if (errorcode!=XpmSuccess) {
-	    use_radar = 0;
+	    USE_RADAR = 0;
 	    if (errorcode == XpmColorFailed) {
 		Sprintf(buf, "Insufficient colors available to load %s.",appResources.tile_file);
 		X11_raw_print(buf);
@@ -405,7 +432,8 @@ init_tiles(wp)
 	    X11_raw_print("Switching to text-based mode.");
 	    goto tiledone;
 	}
-
+	TILE_PER_COL = tile_image.width / TILE_WIDTH;
+#if 0
 	if (tile_image->height%total_tiles_used != 0) {
 	    char buf[BUFSIZ];
 	    Sprintf(buf,
@@ -422,6 +450,7 @@ init_tiles(wp)
 	tile_count=total_tiles_used;
 	tile_width=tile_image->width;
 	tile_height=tile_image->height/tile_count;
+#endif
     }
 #else
     /* any less than 16 colours makes tiles useless */
@@ -596,7 +625,9 @@ init_tiles(wp)
     values.graphics_exposures = False;
 #if 1
     values.foreground = WhitePixelOfScreen(screen) ^
-	XGetPixel(tile_image, 0, tile_height*glyph2tile[cmap_to_glyph(S_corr)]);
+	XGetPixel(tile_image, 
+                tile_width*(glyph2tile[cmap_to_glyph(S_corr)]%TILE_PER_COL),
+                tile_height*(glyph2tile[cmap_to_glyph(S_corr)]/TILE_PER_COL));
 #else
     values.foreground = ~((unsigned long) 0);
 #endif
@@ -615,11 +646,19 @@ tiledone:
     if (colormap) free((genericptr_t)colormap);
     if (tile_bytes) free((genericptr_t)tile_bytes);
     if (colors) free((genericptr_t)colors);
+/*ITA*/
+    {int i; for(i=0;i<MAX_PXM_SLOTS;i++){ pxm_slot[i].age=0;
+     pxm_slot[i].bg=pxm_slot[i].fg=-99;pxm_slot[i].pixmap=0;} }
+/*ITA*/
 #endif
 
     if (result) {				/* succeeded */
+#if 0
 	map_info->square_height = tile_height;
 	map_info->square_width = tile_width;
+#endif
+	map_info->square_height = TILE_HEIGHT;
+	map_info->square_width = TILE_WIDTH;
 	map_info->square_ascent = 0;
 	map_info->square_lbearing = 0;
     } else {
@@ -969,8 +1008,7 @@ struct map_info_t *map_info;
 
     for (sp = (unsigned short *) map_info->mtype.tile_map->glyphs, i = 0;
 	i < ROWNO*COLNO; sp++, i++)
-
-    *sp = stone;
+      *sp = stone;
 }
 
 /*
@@ -1334,48 +1372,60 @@ map_update(wp, start_row, stop_row, start_col, stop_col, inverted)
 
     dpy = XtDisplay(radar);
 
-    if(use_radar && !radar_flg){
+    if(USE_RADAR){
       int i;
     
-      XtPopup(radar_popup, XtGrabNone);
-      radar_flg = 1;
-
-      num_args = 0;
-      XtSetArg(args[num_args], XtNwidth, &width); ++num_args;
-      XtSetArg(args[num_args], XtNheight, &height); ++num_args;
-
-      XtGetValues(radar, args, num_args);
-      depth = DefaultDepth(dpy, DefaultScreen(dpy));
-      radar_window = XtWindow(radar);
-      radar_pixmap = XCreatePixmap(dpy, radar_window,
-		      width, height, depth);
-      radar_pixmap2 = XCreatePixmap(dpy, radar_window,
-		      width, height, depth);
-
-      num_args = 0;
-      XtSetArg(args[num_args], XtNbackgroundPixmap, radar_pixmap); num_args++;
-      XtSetValues(radar, args, num_args);
-
-      num_args = 0;
-      XtSetArg(args[num_args], XtNcolormap, &cmap); num_args++;
-      XtGetValues(radar, args, num_args);
-
-      gcval.function = GXcopy;
-      for(i=0 ; i<16 ; ++i){
-	XAllocColor(dpy, cmap, &radar_color[i]);
-
-	gcval.foreground = radar_color[i].pixel;
-	radar_gc[i] = XCreateGC(dpy, radar_window, 
-				GCFunction | GCForeground,
-				&gcval);
+      if(flags.radar && !radar_is_popuped){
+	  XtPopup(radar_popup, XtGrabNone);
+	  radar_is_popuped = TRUE;
+      }
+      if(!flags.radar && radar_is_popuped){
+	  XtPopdown(radar_popup);
+	  radar_is_popuped = FALSE;
       }
-      radar_gc_black = radar_gc[RADAR_BLACK];
-      radar_gc_white = radar_gc[RADAR_WHITE];
 
-      XFillRectangle(dpy, radar_pixmap,
-		     radar_gc_black, 0, 0, width, height);
-      XFillRectangle(dpy, radar_pixmap2,
-		     radar_gc_black, 0, 0, width, height);
+      if(!radar_is_initialized){
+	  radar_is_initialized = TRUE;
+
+	  XtRealizeWidget(radar_popup);
+
+	  num_args = 0;
+	  XtSetArg(args[num_args], XtNwidth, &width); ++num_args;
+	  XtSetArg(args[num_args], XtNheight, &height); ++num_args;
+
+	  XtGetValues(radar, args, num_args);
+	  depth = DefaultDepth(dpy, DefaultScreen(dpy));
+	  radar_window = XtWindow(radar);
+	  radar_pixmap = XCreatePixmap(dpy, radar_window,
+				       width, height, depth);
+	  radar_pixmap2 = XCreatePixmap(dpy, radar_window,
+					width, height, depth);
+	  
+	  num_args = 0;
+	  XtSetArg(args[num_args], XtNbackgroundPixmap, radar_pixmap); num_args++;
+	  XtSetValues(radar, args, num_args);
+	  
+	  num_args = 0;
+	  XtSetArg(args[num_args], XtNcolormap, &cmap); num_args++;
+	  XtGetValues(radar, args, num_args);
+	  
+	  gcval.function = GXcopy;
+	  for(i=0 ; i<16 ; ++i){
+	      XAllocColor(dpy, cmap, &radar_color[i]);
+	      
+	      gcval.foreground = radar_color[i].pixel;
+	      radar_gc[i] = XCreateGC(dpy, radar_window, 
+				      GCFunction | GCForeground,
+				      &gcval);
+	  }
+	  radar_gc_black = radar_gc[RADAR_BLACK];
+	  radar_gc_white = radar_gc[RADAR_WHITE];
+
+	  XFillRectangle(dpy, radar_pixmap,
+			 radar_gc_black, 0, 0, width, height);
+	  XFillRectangle(dpy, radar_pixmap2,
+			 radar_gc_black, 0, 0, width, height);
+      }
     }
 #endif
 
@@ -1400,22 +1450,101 @@ map_update(wp, start_row, stop_row, start_col, stop_col, inverted)
 	int cur_col;
 	Display* dpy=XtDisplay(wp->w);
 	Screen* screen=DefaultScreenOfDisplay(dpy);
-
+/*ITA*/
+         /* each slots ages */
+        {
+	     int i; 
+	     
+	     for(i=0 ; i<MAX_PXM_SLOTS ; i++)
+		  pxm_slot[i].age++;
+	}
+/*ITA*/
 	for (row = start_row; row <= stop_row; row++) {
 	    for (cur_col = start_col; cur_col <= stop_col; cur_col++) {
+		struct rm *lev = &levl[cur_col][row];
 		int glyph = tile_map->glyphs[row][cur_col];
-		int tile = glyph2tile[glyph];
-		int dest_x = cur_col * map_info->square_width;
-		int dest_y = row * map_info->square_height;
+		int bg = back_to_glyph(cur_col, row);
+		int tile = 0;
+		int bgtile = 0;
+		int dest_x = 0;
+		int dest_y = 0;
+		int src_x;
+		int src_y;
+		int bgsrc_x;
+		int bgsrc_y;
+		
+		if(tile_pixmap){
+		  if(Blind || (viz_array && !cansee(cur_col, row)))
+		      bg = lev->glyph;
 
-		XCopyArea(dpy, tile_pixmap, XtWindow(wp->w),
-			tile_map->black_gc,	/* no grapics_expose */
-			0,
-			tile * map_info->square_height,
-			tile_width, tile_height,
-			dest_x,dest_y);
+		  bgtile = glyph2tile[bg];
+		  tile = glyph2tile[glyph];
+		  dest_x = cur_col * map_info->square_width;
+		  dest_y = row * map_info->square_height;
+		  bgsrc_x = (bgtile % TILE_PER_COL) * TILE_WIDTH;
+		  bgsrc_y = (bgtile / TILE_PER_COL) * TILE_HEIGHT;
+		  src_x = (tile % TILE_PER_COL) * TILE_WIDTH;
+		  src_y = (tile / TILE_PER_COL) * TILE_HEIGHT;
+/*ITA*/
+   {
+     int i, match;
+     int maxage = 0;
+
+     if(bgtile != -1){
+       match = -1;
+       for(i=0 ; i<MAX_PXM_SLOTS ; i++){
+	 if(tile== pxm_slot[i].fg && bgtile== pxm_slot[i].bg){
+	   match = i;
+	   break;
+	 }
+       }
+       if(match == -1){
+           /* no match found:dispose the oldest slot and compose pixmap */
+	 for(i=0 ; i<MAX_PXM_SLOTS ; i++)
+           if(maxage < pxm_slot[i].age){
+	     match = i;
+	     maxage = pxm_slot[i].age;
+	   }
+           if(!pxm_slot[match].pixmap) 
+	     pxm_slot[match].pixmap = XCreatePixmap(
+		 dpy, XtWindow(toplevel),
+		 TILE_WIDTH, TILE_HEIGHT, DefaultDepth(dpy, DefaultScreen(dpy)));
+           XCopyArea(dpy, tile_pixmap, pxm_slot[match].pixmap,
+		     tile_map->black_gc,
+		     bgsrc_x, bgsrc_y,
+		     TILE_WIDTH, TILE_HEIGHT,
+		     0,0);
+
+           XSetClipOrigin(dpy,tile_gc, 0 - src_x, 0 - src_y);
+
+           XCopyArea(dpy, tile_pixmap, pxm_slot[match].pixmap,
+                              tile_gc,
+                              src_x, src_y,
+                              TILE_WIDTH, TILE_HEIGHT,
+                              0,0);
+           pxm_slot[match].fg=tile;
+           pxm_slot[match].bg=bgtile;
+       }
+    /* slot ready */
+       pxm_slot[match].age=0;
+       XCopyArea(dpy, pxm_slot[match].pixmap, XtWindow(wp->w),
+		 tile_map->black_gc,
+		 0,0,
+		 TILE_WIDTH, TILE_HEIGHT,
+		 dest_x,dest_y);
+     }
+     else
+    /* no clip mask */
+	 XCopyArea(dpy, tile_pixmap, XtWindow(wp->w),
+		   tile_map->black_gc,
+		   src_x, src_y,
+		   TILE_WIDTH, TILE_HEIGHT,
+		   dest_x,dest_y);
+}
+/*ITA*/
+}
 #ifdef RADAR
-		if(use_radar){
+		if(tile_pixmap && USE_RADAR){
 #define RADAR_MONSTER	RADAR_YELLOW
 #define RADAR_HUMAN	RADAR_WHITE
 #define RADAR_OBJECT	RADAR_BLUE
@@ -1510,7 +1639,7 @@ map_update(wp, start_row, stop_row, start_col, stop_col, inverted)
 		map_info->square_height-1);
 	}
 #ifdef RADAR
-	if(use_radar){
+	if(USE_RADAR){
 	  XCopyArea(dpy, radar_pixmap2, radar_pixmap, radar_gc_black,
 		    0, 0, RADAR_WIDTH, RADAR_HEIGHT, 0, 0);
 	  check_sb(wp);
@@ -1640,10 +1769,19 @@ static char radar_translations[] =
  <Key>Right: scroll(6)\n\
  <Key>Up: scroll(8)\n\
  <Key>Down: scroll(2)\n\
+ <Key>Escape: dismiss_radar()\n\
  <Key>: input()\
 ";
 #endif
 
+void
+dismiss_radar(Widget w, XEvent*ev , String *s, Cardinal*n)
+{
+    XtPopdown(radar_popup);
+    flags.radar = FALSE;
+    radar_is_popuped = FALSE;
+}
+
 /*
  * The map window creation routine.
  */
